      subroutine ctrackingAll    
c     **************************************************
c       tracking particles given in the stack area.
c       During tracking, new particles may be produced
c       and pushed in the stack area.  They are all
c       treated here, until all particles are processed.
c     **************************************************
      implicit none
#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"

      integer moreptcl, jold, icon
      real*8 smallAS/1./   ! electrons < 1GeV cannot make AS at all.
      real*8 u, iwgt	
c     
      external cblkTracking
c
c      ***  do until no more ptcl in stack
      do while (.true.)
c           get one particle from stack
         call cpop(TrackBefMove, moreptcl)
         if(moreptcl .eq. 0) goto 100   !  exit
         call rndsw(jold, 1)            !  specify 1st generator.
         call cifDead(TrackBefMove, icon)  ! generator may be switched to 2
c            AS generation. 

c         if(icon .eq. 0) then
c            thin sampling
            if(TrackBefMove.p.fm.p(4) .lt. Ethin) then
c                do thin sampling
               iwgt = TrackBefMove.p.fm.p(4)/Ethin ! inverse weight
               call rndc(u)
               if(u .gt. iwgt) then
                  icon =2       !  discard this ptcl
               else
                  TrackBefMove.wgt = TrackBefMove.wgt /iwgt
               endif
            endif    
c         endif

         if(ObserveAS .and. icon .ne. 2) then
            if(TrackBefMove.p.code .eq. kelec) then
               if(TrackBefMove.asflag .eq. 0) then
                  if(TrackBefMove.p.fm.p(4) .le. EasWait) then
                     if(TrackBefMove.p.fm.p(4) .gt. smallAS) then ! 95/08/17
                        call cobAS(TrackBefMove)
                     endif
                     TrackBefMove.asflag =1
                  endif
               endif
            elseif(TrackBefMove.p.code .eq. kphoton) then
               if(icon .ne. 0) then
                  if(TrackBefMove.asflag .eq. 0) then
                     if(TrackBefMove.p.fm.p(4) .gt. smallAS) then
                        icon = 0 ! follow until it becomes e-pair
                     endif
                  endif
               endif
            endif
         endif
c        ----------------------------
         if(icon .eq. 0) then
            call cTracking
         endif
      enddo
 100  continue
      end
c -------------------------------------tracking a ptcl
      subroutine cTracking
      implicit none

#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"
#include  "Zobs.h"
#include  "Zobsv.h"
#include  "Zincidentv.h"
#include  "Zcode.h"

c                next is for DEBUG
#include  "Zcondc.h"

      character*70 msg
cc      integer icon

         call cfixMedia   ! 
c           sample interaction length.  The path may
c           be truncated to a shorter one than really sampled.
c           In that case, MoveStat == Truncated.
         call csampIntL
#if DEBUG == 1
         call checkstat('after csampIntl')
#endif
c           Truncate the path if it is too long. Note, the truncated
c           path in the above process may be again truncated to
c           a shorter one. This happens, for example, if ptcl energy
c           is low.  Path inf is reset in IntInfArray.
         call ctruncPath
#if DEBUG == 1
         call checkstat('after ctruncPath')
#endif

c           compute path end inf. including scattering and
c           mag. deflection.  Path end inf is set in MovedTrack
         call ccompPathEnd
#if DEBUG == 1
         call checkstat('after ccompPathEnd')
#endif
c           see if MovedTrack crosses an observation site.
c         (or reaches BorderHeightL)  If so, reset MovedTrack. 
            call cifXObsSite
#if DEBUG == 4
            call checkstat('after cifXObsSite')
#endif

         if(Trace .ne. 0) then
            call cputTrInfo     ! put trace info.
         endif
c     
         if(MoveStat .eq. Truncated) then
c           see if truncation is really ok.  for energy = 0, 
c           if anti ptcl, it should be anihilated, if decayabl,
c           it should be decayed.
            if(MovedTrack.p.fm.p(4) .eq. MovedTrack.p.mass) then
               if(MovedTrack.p.subcode .eq. antip) then
                  MoveStat = ToInteract
               elseif(MovedTrack.p.code .eq. kmuon) then
                  MoveStat = ToInteract
               elseif(MovedTrack.p.code .eq. kpion) then
                  MoveStat = ToInteract
               elseif(MovedTrack.p.code .eq. kkaon) then
                  MoveStat = ToInteract
               endif
            endif
         endif 
         if(MoveStat .eq. Truncated) then
c              stack current ptcl
              call cpush(MovedTrack)
         elseif(MoveStat .eq. ToInteract ) then
            if(Zfirst .eq. 0.) then
               Zfirst = MovedTrack.pos.depth
c                  reset minimum time to reach the obs level (time from this
c                  hight to the obs. level), if no mag. exists until
c                  the first collision point. or ptcl goes streight
               if(mod(HowGeomag, 2) .eq. 1 .or. 
     *             IncidentCopy.p.charge .eq. 0) then
                  call csetMinTime(MovedTrack)
               endif
            endif
            call cinteraction
         elseif(MoveStat .eq. ToBeObserved) then
            if(MovedTrack.p.fm.p(4)-MovedTrack.p.mass 
     *          .gt. KEminObs) then
c                  call only for high energy particles
c                  (at AS generation, some ptcl may have < KEminObs)
               call cobservation(2)
            endif
c               update observation place
            MovedTrack.where = MovedTrack.where + 1
c            if(MovedTrack.where .le. NoOfSites) then
            if(MovedTrack.where .le. EndLevel) then  ! 96.09.18
c                  if more obs-site, stack current ptcl.
               call cpush(MovedTrack)
            endif
         elseif(MoveStat .eq. BorderL) then
            call cobservation(3)
         elseif(MoveStat .eq. BorderH) then
            call cobservation(1)
         elseif(MoveStat .eq. Dead) then   
c              nothing to do.
         else
            write(msg, *) ' movestat=',MoveStat,' invalid'
           call cerrorMsg(msg, 0)
         endif   
      end
#if DEBUG > 0
      subroutine checkstat(str)
      implicit none

#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"
#include  "Zobs.h"
#include  "Zobsv.h"
      character*(*) str
      integer i

c         if(TrackBefMove.p.code .eq. 6
c     *     .and. TrackBefMove.p.charge .eq. -1) then
            write(*,*) str, ' code=',TrackBefMove.p.code,
     *         ' cg',TrackBefMove.p.charge,
     *         ' e=', TrackBefMove.p.fm.p(4), ' stat=',MoveStat
            write(*, *) ' where', TrackBefMove.where
            write(*,*)  ' h==', TrackBefMove.pos.height,
     *       ' d=', TrackBefMove.pos.depth
c            write(*,*) ' cos=',TrackBefMove.vec.coszenith
c            do i = 1, NumberOfInte
c               write(*, *) ' process=',IntInfArray(i).process
c               write(*, *) ' thickness=',IntInfArray(i).thickness
c               write(*, *) ' length=',IntInfArray(1).length
c            enddo
            write(*,*)  '--------------'
c         endif
      end
#endif
c-------------------------------------------------------
      subroutine cfixMedia
      implicit none
#include "Ztrack.h"
#include "Ztrackv.h"
#include "Ztrackp.h"
#include "Zobs.h"
#include "Zobsp.h"
#include "Zobsv.h"

      real*8 x/0./, y/0./, z/0./
      save x, y, z
      real*8 distant
      integer icon
      real*8 u

      TargetMassN = 14.5
      TargetAtomicN = 7.25
      call rndc(u)
      if(u .lt. .78) then
c          nitrogen
         TargetNucleonNo = 14
         TargetProtonNo = 7
      elseif(u .lt. .99) then
c             oxigen
         TargetNucleonNo = 16
         TargetProtonNo = 8
      else
c         argon
         TargetNucleonNo = 40
         TargetProtonNo = 18
      endif

      if(HowGeomag .le. 2) then
c             distant; change of B is < 1 % 
         call clengSmallBC(TrackBefMove, distant)

         if( (TrackBefMove.pos.xyz.r(1) - x)**2 + 
     *        (TrackBefMove.pos.xyz.r(2) - y)**2 + 
     *        (TrackBefMove.pos.xyz.r(3) - z)**2 
     *        .gt. distant**2) then
c              if more than MagChgDist m from previous mag set.,
c               reset mag field.
            call cgeomag(YearOfGeomag,  TrackBefMove.pos.xyz, 
     *                Mag, icon)
            call ctransMagTo('xyz', TrackBefMove.pos.xyz, 
     *        Mag, Mag)
            x = TrackBefMove.pos.xyz.r(1)
            y = TrackBefMove.pos.xyz.r(2)
            z = TrackBefMove.pos.xyz.r(3)
c//////////
c            write(*, *) sngl(Mag.x), sngl(Mag.y), sngl(Mag.z),
c     *      sngl(TrackBefMove.pos.height), 
c     *      sngl(sqrt(Mag.x**2+ Mag.y**2 + Mag.z**2))
c //////////
         endif
      else
         Mag = MagfieldXYZ
      endif
      end

c     --------------------------------------------------
      subroutine csampIntL
c     **************************************************
c          
c          sample interaction length 
c          and interaction type.
c       Method.  Sample interaction lengths for all possible 
c                interactions (for collisions, in kg/m2, for decay
c                in m).  They are stored in a record /intinf/ IntInfArray
c                given in Ztrackv.h;  
c            In the routine,  cfixProc, 
c                we take the minimum of values given in kg/m2, and
c                convert it in real lenght (m).  In this process,
c                path truncation may occur (if the particle is
c                going upward, and there is very thin air there,
c                then the given thickness may not be realized.
c                Or if the lenght is too large and accuracy 
c                of convesion is not enough due to the earch
c                curverture)
c                If the decay process exists, we compare the length
c                given by the above treatment, and take shorter one.
c                If decay length is shorter,  we assume the decay
c                takes place, else some collision takes place if
c                the path is not truncated. In the latter case,
c                if the path is truncated, MoveStat == Truncated
c                is set.
      implicit none

#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Ztrackp.h"

      real*8 leng/1.d50/   ! will  be truncated  by ctruncPath

c     **************************************************
      call ciniSmpIntL   ! init for interaction length sampling
c
      if(Reverse .ne. 0) then
         call csetIntInf(leng, .true., 'none')
         ProcessNo = 1
      else
         if(TrackBefMove.p.code .eq. kelec) then
            call csampEIntL
         elseif(TrackBefMove.p.code .eq. kphoton) then
            call csampGIntL
         else
            call csampNEPIntL   ! non Electron Photon  Interaction.
         endif
         call cfixProc
         if(.not. Freec .and. Zfirst .eq. 0.) then
            IntInfArray(ProcessNo).length = 0.
            IntInfArray(ProcessNo).thickness = 0.
         endif
      endif
      end
c     **************************** cfixProc **********
      subroutine cfixProc
      implicit none

#include  "Zglobalc.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
#include  "Zearth.h"


      real*8 h, leng, t, minlen, clen2thick
c     
      integer i, jcut
c     
      minlen = Infty

c              get vertical height
       h = TrackBefMove.pos.height
       do i = 1, NumberOfInte
         if(.not. IntInfArray(i).decay) then
c              convert kg/m2 into length
            if(NumberOfInte .eq. 1 .or.
     *        IntInfArray(i).thickness .ne. Infty) then
               call cthick2len(h, TrackBefMove.vec.coszenith, 
     *         IntInfArray(i).thickness, leng, t, jcut)
            else
               leng = Infty
            endif
         else
            leng = IntInfArray(i).length
            jcut = 0
         endif
         if(leng .lt. minlen) then
               if(jcut .ne. 0) then
                  MoveStat = Truncated
               else
                  jcut = 0
                  MoveStat = ToInteract
               endif
               ProcessNo = i
               IntInfArray(i).length = leng
               minlen = leng
         endif
      enddo
      if(IntInfArray(ProcessNo).decay .or. jcut .ne. 0) then
         IntInfArray(ProcessNo).thickness = clen2thick(h, 
     *      TrackBefMove.vec.coszenith, 
     *      IntInfArray(ProcessNo).length )
      endif
      end
c     ***********************
c          truncate a path of the particle, if the path
c        given in InfIntArray is too long, or the path traverses
c        an observation depth.
      subroutine ctruncPath
      implicit none

#include  "Ztrack.h"
#include  "Ztrackv.h"

      real*8 leng, thick
c
      call cmaxMovLen(leng, thick)
      if(leng .lt. IntInfArray(ProcessNo).length) then
         MoveStat = Truncated
         IntInfArray(ProcessNo).length = leng
         IntInfArray(ProcessNo).thickness  = thick
      endif
      end
